#region Disclaimer by Teo Lachev "Applied Microsoft SQL Server 2008 Reporting Services"
/*============================================================================
  File:     Program.cs

  Summary:  Demonstrates using RDLOM to generate report definitions programatically. 
--------------------------------------------------------------------
  This code sample uses the Microsoft.ReportingServices.RdlObjectModel assembly,
  which represents the Microsoft RDL Object Model. The Microsoft RDL Object Model
  is an internal and unsupported component of Reporting Services. As such, Microsoft may 
  change or discontinue RDLOM. Use RDLOM at your own risk.
===========================================================================*/
#endregion
using System;
using System.IO;
using Microsoft.ReportingServices.RdlObjectModel;
using Microsoft.ReportingServices.RdlObjectModel.Serialization;

namespace Prologika.RDLOM
{
    class Program
    {
        static void Main(string[] args)
        {
            Report report = null;
            RdlSerializer serializer = new RdlSerializer();           
            // Generate a report def and save it to disk
            GenerateTablixReport();

            // Deserialize the report from disk
            if (!File.Exists(ReportPath)) return;
            using (FileStream fs = File.OpenRead(ReportPath))
            {
                serializer = new RdlSerializer();
                report = serializer.Deserialize(fs);
            }
            report.Author = "Teo Lachev";
            report.Description = "RDL Object Demo";
            // TODO: use and abuse RDL as you wish

            // Save the changes
            using (FileStream os = new FileStream(ReportPath, FileMode.Create))
            {
                serializer.Serialize(os, report);
            }
        }
        /// <summary>
        /// Returns the application startup path
        /// </summary>
        private static string ReportPath
        {
            get
            {
                return String.Format(@"{0}\TablixDemo.rdl", System.IO.Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().Location));
            }
        }
        static void GenerateTablixReport()
        {
            RdlSerializer serializer = new RdlSerializer(); 

            TablixMember member;
            TablixColumn column;
            TablixRow row;
            Field f;

            Report report = new Report();

            // Report-level properties
            report.Language = "en-US";
            report.Width = new ReportSize(6.5);
            report.Author = "Teo Lachev";
            report.Page.LeftMargin = report.Page.TopMargin = report.Page.RightMargin = report.Page.BottomMargin = new ReportSize(.5);
            // Data source
            DataSource dataSource = new DataSource();
            dataSource.Name = "AdventureWorks2008";
            dataSource.ConnectionProperties = new ConnectionProperties();
            dataSource.ConnectionProperties.ConnectString = "data source=localhost;initial catalog=AdventureWorks2008;";
            dataSource.ConnectionProperties.DataProvider = "SQL";
            dataSource.ConnectionProperties.IntegratedSecurity = true; 
            //dataSource.DataSourceReference = "AdventureWorks2008"; //for shared data source
            report.DataSources.Add(dataSource);
            // Dataset
            DataSet ds = new DataSet();
            ds.Name = "Main";
            ds.Query = new Query();
            ds.Query.CommandText = "SELECT ProductID, Name, ProductNumber, StandardCost FROM Production.Product WHERE StandardCost > 0 ORDER BY Name";
            ds.Query.DataSourceName = "AdventureWorks2008";
            f = new Field();
            f.Name = "ProductID";
            f.DataField = "ProductID";
            ds.Fields.Add(f);
            f = new Field();
            f.Name = "Name";
            f.DataField = "Name";
            ds.Fields.Add(f);
            f = new Field();
            f.Name = "ProductNumber";
            f.DataField = "ProductNumber";
            ds.Fields.Add(f);
            f = new Field();
            f.Name = "StandardCost";
            f.DataField = "StandardCost";
            ds.Fields.Add(f);
            report.DataSets.Add(ds);

            // Report Body
            report.Body.Height = new ReportSize("6.5");
            // Tablix
            Tablix tablix = new Tablix();
            tablix.Name = "tblMain";
            report.Body.ReportItems.Add(tablix);
            tablix.Top = new ReportSize(.5);
            tablix.DataSetName = "Main";
            // Table columns
            tablix.TablixColumnHierarchy = new TablixHierarchy();
            member = new TablixMember();
            tablix.TablixColumnHierarchy.TablixMembers.Add(member);
            column = new TablixColumn();
            column.Width = new ReportSize(1);
            tablix.TablixBody.TablixColumns.Add(column);
            member = new TablixMember();
            tablix.TablixColumnHierarchy.TablixMembers.Add(member);
            column = new TablixColumn();
            column.Width = new ReportSize(2);
            tablix.TablixBody.TablixColumns.Add(column);
            member = new TablixMember();
            tablix.TablixColumnHierarchy.TablixMembers.Add(member);
            column = new TablixColumn();
            column.Width = new ReportSize(1);
            tablix.TablixBody.TablixColumns.Add(column);
            member = new TablixMember();
            tablix.TablixColumnHierarchy.TablixMembers.Add(member);
            column = new TablixColumn();
            column.Width = new ReportSize(1);
            tablix.TablixBody.TablixColumns.Add(column);
                
            // Table header
            member = new TablixMember();
            tablix.TablixRowHierarchy = new TablixHierarchy();
            tablix.TablixRowHierarchy.TablixMembers.Add(member);
            row = new TablixRow();
            tablix.TablixBody.TablixRows.Add(row);
            row.Height = new ReportSize(.25);

            TablixCell cell = new TablixCell();
            row.TablixCells.Add(cell);
            cell.CellContents = new CellContents();
            cell.CellContents.ReportItem = new Textbox();
            ((Textbox) cell.CellContents.ReportItem).Name = "r0t0";
            ((Textbox) cell.CellContents.ReportItem).Paragraphs[0].TextRuns[0].Value = new ReportExpression("ProductID");
            ((Textbox)cell.CellContents.ReportItem).Style = new Style();
            ((Textbox)cell.CellContents.ReportItem).Style.PaddingRight = new ReportExpression<ReportSize>("2pt");
            ((Textbox) cell.CellContents.ReportItem).Paragraphs[0].Style = new Style();
            ((Textbox)cell.CellContents.ReportItem).Paragraphs[0].Style.TextAlign = new ReportExpression<TextAlignments>("Right"); 
            ((Textbox)cell.CellContents.ReportItem).Paragraphs[0].TextRuns[0].Style = new Style();
            ((Textbox)cell.CellContents.ReportItem).Paragraphs[0].TextRuns[0].Style.FontWeight = new ReportExpression<FontWeights>("Bold");

            cell = new TablixCell();
            row.TablixCells.Add(cell);
            cell.CellContents = new CellContents();
            cell.CellContents.ReportItem = new Textbox();
            ((Textbox)cell.CellContents.ReportItem).Name = "r0t1";
            ((Textbox)cell.CellContents.ReportItem).Paragraphs[0].TextRuns[0].Value = new ReportExpression("Name");
            ((Textbox)cell.CellContents.ReportItem).Style = new Style();
            ((Textbox)cell.CellContents.ReportItem).Style.PaddingRight = new ReportExpression<ReportSize>("2pt");
            ((Textbox)cell.CellContents.ReportItem).Paragraphs[0].Style = new Style();
            ((Textbox)cell.CellContents.ReportItem).Paragraphs[0].TextRuns[0].Style = new Style();
            ((Textbox)cell.CellContents.ReportItem).Paragraphs[0].TextRuns[0].Style.FontWeight = new ReportExpression<FontWeights>("Bold");

            cell = new TablixCell();
            row.TablixCells.Add(cell);
            cell.CellContents = new CellContents();
            cell.CellContents.ReportItem = new Textbox();
            ((Textbox)cell.CellContents.ReportItem).Name = "r0t2";
            ((Textbox)cell.CellContents.ReportItem).Paragraphs[0].TextRuns[0].Value = new ReportExpression("Number");
            ((Textbox)cell.CellContents.ReportItem).Style = new Style();
            ((Textbox)cell.CellContents.ReportItem).Style.PaddingRight = new ReportExpression<ReportSize>("2pt");
            ((Textbox)cell.CellContents.ReportItem).Paragraphs[0].Style = new Style();
            ((Textbox)cell.CellContents.ReportItem).Paragraphs[0].TextRuns[0].Style = new Style();
            ((Textbox)cell.CellContents.ReportItem).Paragraphs[0].TextRuns[0].Style.FontWeight = new ReportExpression<FontWeights>("Bold");

            cell = new TablixCell();
            row.TablixCells.Add(cell);
            cell.CellContents = new CellContents();
            cell.CellContents.ReportItem = new Textbox();
            ((Textbox)cell.CellContents.ReportItem).Name = "r0t3";
            ((Textbox)cell.CellContents.ReportItem).Paragraphs[0].TextRuns[0].Value = new ReportExpression("Cost");
            ((Textbox)cell.CellContents.ReportItem).Style = new Style();
            ((Textbox)cell.CellContents.ReportItem).Style.PaddingRight = new ReportExpression<ReportSize>("2pt");
            ((Textbox)cell.CellContents.ReportItem).Paragraphs[0].Style = new Style();
            ((Textbox)cell.CellContents.ReportItem).Paragraphs[0].Style.TextAlign = new ReportExpression<TextAlignments>("Right"); 
            ((Textbox)cell.CellContents.ReportItem).Paragraphs[0].TextRuns[0].Style = new Style();
            ((Textbox)cell.CellContents.ReportItem).Paragraphs[0].TextRuns[0].Style.FontWeight = new ReportExpression<FontWeights>("Bold"); ;

            //Table details
            member = new TablixMember();
            tablix.TablixRowHierarchy.TablixMembers.Add(member);
            member.Group = new Group();
            member.Group.Name = "Tablix1_DetailsGroup";
            row = new TablixRow();
            tablix.TablixBody.TablixRows.Add(row);
            row.Height = new ReportSize(.25);

            cell = new TablixCell();
            row.TablixCells.Add(cell);
            cell.CellContents = new CellContents();
            cell.CellContents.ReportItem = new Textbox();
            ((Textbox)cell.CellContents.ReportItem).Name = "r1t0";
            ((Textbox)cell.CellContents.ReportItem).Paragraphs[0].TextRuns[0].Value = new ReportExpression("=Fields!ProductID.Value");

            ((Textbox)cell.CellContents.ReportItem).Style = new Style();
            ((Textbox)cell.CellContents.ReportItem).Style.PaddingRight = new ReportExpression<ReportSize>("2pt"); 
            ((Textbox)cell.CellContents.ReportItem).Paragraphs[0].Style = new Style();
            ((Textbox)cell.CellContents.ReportItem).Paragraphs[0].Style.TextAlign = new ReportExpression<TextAlignments>("Right");
            ((Textbox)cell.CellContents.ReportItem).Paragraphs[0].TextRuns[0].Style = new Style();

            cell = new TablixCell();
            row.TablixCells.Add(cell);
            cell.CellContents = new CellContents();
            cell.CellContents.ReportItem = new Textbox();
            ((Textbox)cell.CellContents.ReportItem).Name = "r1t1";
            ((Textbox)cell.CellContents.ReportItem).Paragraphs[0].TextRuns[0].Value = new ReportExpression("=Fields!Name.Value");
            ((Textbox)cell.CellContents.ReportItem).Style = new Style();
            ((Textbox)cell.CellContents.ReportItem).Style.PaddingRight = new ReportExpression<ReportSize>("2pt");
            ((Textbox)cell.CellContents.ReportItem).Paragraphs[0].Style = new Style();

            cell = new TablixCell();
            row.TablixCells.Add(cell);
            cell.CellContents = new CellContents();
            cell.CellContents.ReportItem = new Textbox();
            ((Textbox)cell.CellContents.ReportItem).Name = "r1t2";
            ((Textbox)cell.CellContents.ReportItem).Paragraphs[0].TextRuns[0].Value = new ReportExpression("=Fields!ProductNumber.Value");
            ((Textbox)cell.CellContents.ReportItem).Style = new Style();
            ((Textbox)cell.CellContents.ReportItem).Style.PaddingRight = new ReportExpression<ReportSize>("2pt");
            ((Textbox)cell.CellContents.ReportItem).Paragraphs[0].Style = new Style();

            cell = new TablixCell();
            row.TablixCells.Add(cell);
            cell.CellContents = new CellContents();
            cell.CellContents.ReportItem = new Textbox();
            ((Textbox)cell.CellContents.ReportItem).Name = "r1t3";
            ((Textbox)cell.CellContents.ReportItem).Paragraphs[0].Style = new Style();
            ((Textbox)cell.CellContents.ReportItem).Paragraphs[0].Style.TextAlign = new ReportExpression<TextAlignments>("Right");
            ((Textbox)cell.CellContents.ReportItem).Paragraphs[0].TextRuns[0].Value = new ReportExpression("=Fields!StandardCost.Value");
            ((Textbox)cell.CellContents.ReportItem).Style = new Style();
            ((Textbox)cell.CellContents.ReportItem).Style.PaddingRight = new ReportExpression<ReportSize>("2pt");

            
            // serialize to disk
            using (FileStream os = new FileStream(ReportPath, FileMode.Create))
            {
                serializer.Serialize(os, report);
            }
        }
    }
}
