using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Configuration;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Text;
using System.Windows.Forms;
//using ReportingSetup.ReportingService;
using ReportingSetup.WSS;

namespace ReportingSetup
{
    public enum ReportType
    {
        StandardReport,
        ListingReport
    }

    public partial class ReportingConfigurationForm : Form
    {
        private ReportingService2006 _reportService;
        private string _rootFolder = ConfigurationManager.AppSettings["RootFolder"];
        private string[] _reports = Directory.GetFiles(Application.StartupPath, "*.rdl");
        private string _folder = null;

        public ReportingConfigurationForm()
        {
            InitializeComponent();

            _reportService = new ReportingService2006();
            _reportService.UseDefaultCredentials = true;
            txtURL.Text = ReportingSetup.Properties.Settings.Default.ReportingSetup_WSS_ReportingService2006;
            _reportService.Url = txtURL.Text;
            _folder = txtFolder.Text;
        }

        private void btnCreate_Click(object sender, EventArgs e)
        {
            try
            {
                Cursor.Current = Cursors.WaitCursor;

                if (_reports.Length > 0)
                {
                    txtOutput.Text = string.Empty;
                    txtOutput.Text += "Cleanup" + Environment.NewLine;
                    txtOutput.Text += "*****************************************************************************************" + Environment.NewLine;

                    try
                    {
                        _reportService.DeleteItem(String.Format("{0}/{1}", _rootFolder, _folder));
                    }
                    catch (System.Exception ex)
                    {
                        //do nothing 
                    }
                    txtOutput.Text += String.Format("Deleting Folder :             /{0}", _folder) + Environment.NewLine;
                    txtOutput.Text += Environment.NewLine;
                }
                this.Refresh();
                System.Threading.Thread.Sleep(100);

                UploadContent();
            }
            catch (Exception ex)
            {
                txtOutput.Text += ex.Message;
            }
            finally
            {
                Cursor.Current = Cursors.Default;
            }
        }

        private void UploadContent()
        {
            txtOutput.Text += "Folders" + Environment.NewLine;
            txtOutput.Text += "*****************************************************************************************" + Environment.NewLine;
            this.Refresh();
            System.Threading.Thread.Sleep(100);

            //--- Folders
            CreateFolder(_folder, String.Format("{0}/", _rootFolder));

            txtOutput.Text += Environment.NewLine + "DataSources" + Environment.NewLine;
            txtOutput.Text += "*****************************************************************************************" + Environment.NewLine;
            this.Refresh();
            System.Threading.Thread.Sleep(100);

            //--- Datasource
            CreateSQLDataSource("AdventureWorks", String.Format("{0}/{1}/", _rootFolder, _folder));
            CreateASDataSource("AdventureWorksAS", String.Format("{0}/{1}/", _rootFolder, _folder));
            
            txtOutput.Text += Environment.NewLine + "Reports" + Environment.NewLine;
            txtOutput.Text += "*****************************************************************************************" + Environment.NewLine;
            this.Refresh();
            System.Threading.Thread.Sleep(100);

            //--- Custom Reports
            if (_reports.Length > 0)
            {
                foreach (string customReport in _reports)
                {
                    CreateReport(customReport, String.Format("{0}/{1}/", _rootFolder, _folder));
                }
            }

        }
        private void CreateSQLDataSource(string datasource, string parent)
        {
            txtOutput.Text += "Creating DataSource :    " + GetPath(parent + "/" + datasource) + Environment.NewLine;
            this.Refresh();
            System.Threading.Thread.Sleep(100);

            DataSourceDefinition dsDefinition = new DataSourceDefinition();

            //--- Set the DataSource properties
            dsDefinition.Extension = "SQL";
            dsDefinition.ConnectString = "Data Source=localhost;Initial Catalog=AdventureWorks";
            dsDefinition.UseOriginalConnectString = false;
            dsDefinition.OriginalConnectStringExpressionBased = false;
            dsDefinition.CredentialRetrieval = CredentialRetrievalEnum.Integrated;
            dsDefinition.ImpersonateUserSpecified = false;
            //Use the default prompt string.
            dsDefinition.Prompt = null;
            dsDefinition.WindowsCredentials = false;
            dsDefinition.Enabled = true;
            dsDefinition.EnabledSpecified = true;

            //--- Create the DataSource
            _reportService.CreateDataSource(datasource + ".rsds", parent, true, dsDefinition, null);
        }
        private void CreateASDataSource(string datasource, string parent)
        {
            txtOutput.Text += "Creating DataSource :    " + GetPath(parent + "/" + datasource) + Environment.NewLine;
            this.Refresh();
            System.Threading.Thread.Sleep(100);

            DataSourceDefinition dsDefinition = new DataSourceDefinition();

            //--- Set the DataSource properties
            dsDefinition.Extension = "OLEDB-MD";
            dsDefinition.ConnectString = "Data Source=localhost;Initial Catalog=IPSUDM";
            dsDefinition.UseOriginalConnectString = false;
            dsDefinition.OriginalConnectStringExpressionBased = false;
            dsDefinition.CredentialRetrieval = CredentialRetrievalEnum.Integrated;
            dsDefinition.ImpersonateUserSpecified = false;
            //Use the default prompt string.
            dsDefinition.Prompt = null;
            dsDefinition.WindowsCredentials = false;
            dsDefinition.Enabled = true;
            dsDefinition.EnabledSpecified = true;

            //--- Create the DataSource
            _reportService.CreateDataSource(datasource + ".rsds", parent, true, dsDefinition, null);
        }


        private void CreateFolder(string folder, string parent)
        {
            txtOutput.Text += "Creating Folder :             " + GetPath(parent + "/" + folder) + Environment.NewLine;
            this.Refresh();
            System.Threading.Thread.Sleep(100);

            //--- Create the Folder
            //if(!ItemExists(GetPath(parent + "/" + folder)))
            _reportService.CreateFolder(folder, parent);
            //else
            //    txtOutput.Text += "* Folder " + GetPath(parent + "/" + folder) + " already exists." + Environment.NewLine;

            this.Refresh();
            System.Threading.Thread.Sleep(100);
        }

        private void CreateReport(string report, string parent)
        {
            report = Path.GetFileName(report);
            Warning[] warnings = null;

            txtOutput.Text += "Creating Report :             " + GetPath(parent + "/" + report) + Environment.NewLine;
            this.Refresh();
            System.Threading.Thread.Sleep(100);

            byte[] reportBytes = null;
            string reportPath = report; 

            //--- Create the Report
            if (File.Exists(reportPath))
            {
                reportBytes = File.ReadAllBytes(reportPath);
                _reportService.CreateReport(report, parent, true, reportBytes, null, out warnings);

               }
            else
                txtOutput.Text += string.Format("* Report {0} not found in {1}.", reportPath, Path.GetDirectoryName(Process.GetCurrentProcess().MainModule.FileName) + "\\Reports") + Environment.NewLine;

            this.Refresh();
            System.Threading.Thread.Sleep(100);
        }

        private void CreateResource(string resource, string parent, string mimeType)
        {
            txtOutput.Text += "Creating Resource :             " + GetPath(parent + "/" + resource) + Environment.NewLine;
            this.Refresh();
            System.Threading.Thread.Sleep(100);

            byte[] resourceBytes = null;
            string resourcePath = "Resources\\" + resource;

            //--- Create the Resource
            if (File.Exists(resourcePath))
            {       
                resourceBytes = File.ReadAllBytes(resourcePath);
                _reportService.CreateResource(resource, parent, true, resourceBytes, mimeType, null);
            }
            else
                txtOutput.Text += string.Format("* Resource {0} not found in {1}.", resourcePath, Path.GetDirectoryName(Process.GetCurrentProcess().MainModule.FileName)) + Environment.NewLine;

            this.Refresh();
            System.Threading.Thread.Sleep(100);
        }

        private string GetFolderName(string itemPath)
        {
            string folderPath = Path.GetDirectoryName(itemPath);
            folderPath = folderPath.Replace(@"\", "/");
            return folderPath;
        }

        private string GetPath(string path)
        {
            if(path.StartsWith("//"))
                return path.Substring(1);

            return path;
        }

 
    }
}